package flutter.fl_print_pl

import android.content.Context
import android.util.Log
import android.widget.Toast
import androidx.annotation.NonNull
import com.google.gson.Gson
import com.google.gson.JsonArray
import com.google.gson.JsonParser
import com.inuker.bluetooth.library.BluetoothClient
import com.inuker.bluetooth.library.search.SearchRequest
import com.inuker.bluetooth.library.search.SearchResult
import com.inuker.bluetooth.library.search.response.SearchResponse
import cpcl.Config
import cpcl.PrinterHelper
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.EventChannel.EventSink
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import org.json.JSONArray
import org.json.JSONObject


/** FlPrintPlPlugin*每次修改代码 要点红停止，重新运行*/
public class FlPrintPlPlugin : FlutterPlugin, MethodCallHandler {
    private var context: Context? = null
    private var methodChannel: MethodChannel? = null
    private lateinit var eventChannel: EventChannel
    private lateinit var mBluetoothClient: BluetoothClient
    val mListData = mutableListOf<String>()
    private var eventSink: EventSink? = null
    private var listPrintData = mutableListOf<PrintDataEntity>()
    private var listOrder = mutableListOf<String>()
    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        //测试获取上下文
        context = flutterPluginBinding.applicationContext
        mBluetoothClient = BluetoothClient(context)
        //通信方法注册
        methodChannel = MethodChannel(flutterPluginBinding.binaryMessenger, "fl_print_pl").apply {
            setMethodCallHandler(this@FlPrintPlPlugin)
        }
        //主动传输注册
        eventChannel = EventChannel(flutterPluginBinding.binaryMessenger, "android_print_event_channel").apply {
            setStreamHandler(object : EventChannel.StreamHandler {
                override fun onListen(arguments: Any?, events: EventSink?) {
                    this@FlPrintPlPlugin.eventSink = events
                }

                override fun onCancel(arguments: Any?) {
                    this@FlPrintPlPlugin.eventSink = null
                }
            })
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        methodChannel?.setMethodCallHandler(null)
        methodChannel = null
        mListData.clear()
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        if (call.method == "getPlatformVersion") {
            result.success("Android ${android.os.Build.VERSION.RELEASE}")
        } else if (call.method == "searchBlue") {
            searchBlue()
        } else if (call.method == "sendPrintData") {
            //后台数据返回不同自行调整
            listPrintData.clear()
            val params = call.arguments<Map<String, String>>()
            Log.d("xg", "打印数据${params["JsonData"].toString()}")
            val jsonElements: JsonArray = JsonParser().parse(params["JsonData"].toString().trim()).asJsonArray //获取JsonArray对象
            for (bean in jsonElements) {
                //gosn 解析 release 为空 使用原生解析
                val jsonObject = JSONObject(bean.toString())
                val orderBoxNoList = jsonObject.getJSONArray("orderBoxNoList")
                for (index in 0 until orderBoxNoList.length()) {
                    listOrder.add(orderBoxNoList[index].toString())
                }
                val beanItem = PrintDataEntity(
                        jsonObject.getString("amount"),
                        jsonObject.getString("consignorAddress"),
                        jsonObject.getString("consignorAreaName"),
                        jsonObject.getString("consignorCityName"),
                        jsonObject.getString("consignorName"),
                        jsonObject.getString("consignorPhone"),
                        jsonObject.getString("goodsNum"),
                        jsonObject.getString("goodsNum"),
                        listOrder,
                        jsonObject.getString("orderTime"),
                        jsonObject.getString("platName"),
                        jsonObject.getString("preOrderNo"),
                        jsonObject.getString("receiveAddress"),
                        jsonObject.getString("receiveAreaName"),
                        jsonObject.getString("receiveCityName"),
                        jsonObject.getString("receiveName"),
                        jsonObject.getString("receivePhone"),
                        jsonObject.getString("receiveStoreName"),
                        jsonObject.getString("thirdOrderNo"))
                listPrintData.add(beanItem)
            }
            connectionPrint(params["address"].toString().trim(), listPrintData)
            // 通知 Dart 层
        } else if (call.method == "getBlueData") {
            result.success("$mListData")
        } else {
            result.notImplemented()
        }
    }

    /**
     * 暴露出去供界面传数据到Flutter
     */
    private fun sendEvent(data: Any) {
        if (eventSink != null) {
            eventSink?.success(data)
        } else {
            Log.d("xg", "===== FlutterEventChannel.eventSink 为空 需要检查一下 =====")
        }
    }

    /**
     * 搜索蓝牙
     */
    private fun searchBlue() {
        if (!mBluetoothClient.isBluetoothOpened!!) mBluetoothClient.openBluetooth()
        val request = SearchRequest.Builder()
                .searchBluetoothClassicDevice(5000) // 再扫经典蓝牙5s
                .build()

        mBluetoothClient?.search(request, object : SearchResponse {
            override fun onSearchStarted() {}
            override fun onDeviceFounded(device: SearchResult) {
                Log.d("xg", "蓝牙名字${device.name}")
                Log.d("xg", "蓝牙Id${device.address}")
                if (!device.name.isNullOrBlank()) {
                    if (device.name.startsWith("HM") || device.name.startsWith("BM")) {
                        if (!mListData.contains(device.name)) {
                            mListData.add(device.name.trim())
                            mListData.add(device.address.trim())
                        }
                    }

                }

            }

            override fun onSearchStopped() {}
            override fun onSearchCanceled() {}
        })
    }


    /**
     * 打印箱码
     *   BM Bluetooth,FC:58:FA:34:A7:D2
     */
    private fun connectionPrint(address: String, printDataEntity: MutableList<PrintDataEntity>) {
        Log.d("xg", "蓝牙地址${address.toString()}")
        Log.d("xg", "打印数据printDataEntity${printDataEntity[0].preOrderNo}}")
        Log.d("xg", "数据列表size${printDataEntity.size.toString()}")
        Observable.create<Int> { emitter ->
                    try {
                        Config.isShake = false
                        emitter.onNext(PrinterHelper.PortOpen("Bluetooth,$address"))
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }
                }.subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({ status ->
                    Log.d("xg", "0：连接成功，-1：连接异常，-2：蓝牙地址错误，-3：打印机与 SDK 不匹配（握手指令错误）")
                    Log.d("xg", "打印机连接status${status.toString()}")
                    if (status == 0) {
                        if (onPrint(printDataEntity)) {
                            sendEvent("printSuccess")
                            Toast.makeText(context, "打印数据成功", Toast.LENGTH_SHORT).show()
                        } else {
                            Toast.makeText(context, "打印数据失败", Toast.LENGTH_SHORT).show()
                        }
                    } else {
                        Toast.makeText(context, "打印机连接失败", Toast.LENGTH_SHORT).show()
                    }
                }, {
                    if (it.message !== null) {
                        Toast.makeText(context, "打印数据失败${it.message}", Toast.LENGTH_SHORT).show()
                        Log.d("xg", "打印数据失败${it.message}")
                    }

                })
    }

    /**
     *  todo 打印任务
     */
    private fun onPrint(printDataList: MutableList<PrintDataEntity>): Boolean {
        var isSuccess = true
        printDataList?.forEach { orderInfo ->
            if (orderInfo.consignorAddress.isNullOrBlank()) {
                Log.d("xg", "打印数据为空")
            }
            orderInfo?.apply {
                //先打印运单
                PrinterHelper.printAreaSize("0", "200", "400", "800", "1")
                //550 一共 600 左右留25
                PrinterHelper.InverseLine("25", "0", "600", "0", "15")
                //物流名称
                PrinterHelper.Align(PrinterHelper.LEFT)
                PrinterHelper.SetBold("2")
                PrinterHelper.SetMag("2", "2")//对下面的字体进行放大（如不需要不用添加）
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "100", "30", orderInfo.platName)
                PrinterHelper.SetMag("1", "1")//关闭放大
                //箱码
                PrinterHelper.Align(PrinterHelper.TEXT)
                PrinterHelper.SetBold("1")
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "45", "120", "运单号:")
                PrinterHelper.Align(PrinterHelper.TEXT)
                PrinterHelper.SetBold("1")
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "45", "160", "${orderInfo.preOrderNo}")
                PrinterHelper.SetBold("2")
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "45", "220", "共${(orderInfo.goodsNum)}件  ${AmountUtil.amountFormat(orderInfo.amount.toLong())}元")
                PrinterHelper.SetBold("2")
                PrinterHelper.Align(PrinterHelper.LEFT)
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "45", "260", "寄件人:")
                PrinterHelper.SetBold("1")
                PrinterHelper.Align(PrinterHelper.LEFT)
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "45", "300", "${orderInfo.consignorName}")
                PrinterHelper.SetBold("2")
                PrinterHelper.Align(PrinterHelper.LEFT)
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "45", "340", "下单时间:")
                PrinterHelper.SetBold("1")
                PrinterHelper.Align(PrinterHelper.LEFT)
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "45", "380", "${DateFormatUtil.dateFormat(orderInfo
                        .orderTime)}")
                PrinterHelper.SetBold("2")
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "45", "420", "收件信息:")
                PrinterHelper.Align(PrinterHelper.LEFT)
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "45", "460", "${orderInfo.receiveStoreName}")
                PrinterHelper.SetBold("1")
                PrinterHelper.Align(PrinterHelper.LEFT)
                PrinterHelper.AutLine("45", "500", 500, 4, false, false, "${orderInfo.receiveAddress}")
                //用户签名
                var tempLength = 0
                if (orderInfo.receiveAddress.length > 20) {
                    tempLength = 40
                }
                PrinterHelper.SetBold("2")
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "45", "${(560 + tempLength)}", "用户签名:")
                PrinterHelper.SetBold("2")
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "45", "${(620 + tempLength)}", "迪乐泰订单可进行扫码开单")
                //恢复默认设置
                PrinterHelper.SetMag("1", "1")
                PrinterHelper.SetBold("0")
                PrinterHelper.Align(PrinterHelper.LEFT)
                PrinterHelper.Form()
                isSuccess = isSuccess && PrinterHelper.Print() > 0
            }

            orderInfo?.orderBoxNoList?.forEachIndexed { index, it ->
                PrinterHelper.printAreaSize("0", "200", "400", "800", "1") //高 750 宽1000
                PrinterHelper.SetBold("2")
                //550 一共 600 左右留25
                PrinterHelper.InverseLine("25", "0", "600", "0", "15")
                //物流名称
                PrinterHelper.Align(PrinterHelper.LEFT)
                PrinterHelper.SetBold("2")
                PrinterHelper.SetMag("2", "2")//对下面的字体进行放大（如不需要不用添加）
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "25", "45", "灵猴")
                PrinterHelper.SetMag("1", "1")//关闭放大
                //物流名称
                PrinterHelper.Align(PrinterHelper.LEFT)
                PrinterHelper.SetBold("2")
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "25", "135", "汽配物流")
                //寄件人城市，收件人城市
                PrinterHelper.Align(PrinterHelper.LEFT)
                PrinterHelper.SetBold("2")//加黑
                // 字体大小 PrinterHelper x柱位置 y柱位置
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "170", "55", "${"寄:" + AddressUtil.getCity(orderInfo.consignorAddress)}")
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "330", "55", "--")
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "170", "135", "收:")
                PrinterHelper.SetMag("2", "2")//对下面的字体进行放大（如不需要不用添加）
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "365", "30", "${AddressUtil.getCityArea(orderInfo.consignorAddress)}")
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "250", "110", "${AddressUtil.getCityArea(orderInfo.receiveAddress)}")
                PrinterHelper.SetMag("1", "1")//关闭放大
                PrinterHelper.InverseLine("25", "180", "600", "180", "5")
                //条形码
                PrinterHelper.Align(PrinterHelper.LEFT)
                PrinterHelper.SetBold("0")
                /**
                 *  "3" 28=2.8比例 越大 二维码越宽  "10", 距离右边位置  "200"距离上边位置 "110"高度
                 */
                PrinterHelper.PrintQR(PrinterHelper.BARCODE, "45", "190", "1", "9", it)
                //HPRTPrinterHelper.Barcode(HPRTPrinterHelper.BARCODE, HPRTPrinterHelper.code128, "2", "2", "110", "45", "200", false, "6", "0", "5", it.boxNo)
                //箱码
                PrinterHelper.Align(PrinterHelper.TEXT)
                PrinterHelper.SetBold("1")
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "270", "200", "箱码:")
                PrinterHelper.Align(PrinterHelper.TEXT)
                PrinterHelper.SetBold("1")
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "270", "240", "${it}")
                //件数
                PrinterHelper.Align(PrinterHelper.TEXT)
                PrinterHelper.SetBold("2")
                if (orderInfo.orderBoxNoList.size > 9) {
                    PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "270", "297", "第")
                } else {
                    PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "300", "297", "第")
                }
                PrinterHelper.SetMag("2", "2")//对下面的字体进行放大（如不需要不用添加）
                if (orderInfo.orderBoxNoList.size > 9) {
                    PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "300", "275", "${(index + 1)}")
                } else {
                    PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "347", "275", "${(index + 1)}")
                }

                PrinterHelper.SetMag("1", "1")//
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "380", "297", "件")

                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "430", "297", "共")
                PrinterHelper.SetMag("2", "2")//对下面的字体进行放大（如不需要不用添加）
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "470", "275", "${(orderInfo.orderBoxNoList?.size)}")
                PrinterHelper.SetMag("1", "1")//
                if (orderInfo.orderBoxNoList?.size > 9) {
                    PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "540", "297", "件")
                } else {
                    PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "510", "297", "件")
                }

//              //运单号
                PrinterHelper.Align(PrinterHelper.LEFT)
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "45", "405", "用户订单号:${orderInfo.thirdOrderNo}")// 15 高度
                //运单号
                PrinterHelper.SetBold("2")
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "45", "445", "揽货订单号:${orderInfo.preOrderNo}")
                PrinterHelper.InverseLine("25", "485", "600", "485", "5")

//            //发件信息
                PrinterHelper.Align(PrinterHelper.LEFT)
                PrinterHelper.SetBold("2")
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "45", "500", "发件信息:")
                PrinterHelper.SetBold("1")
                //收件地址
                PrinterHelper.Align(PrinterHelper.LEFT)
                PrinterHelper.SetBold("2")
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "45", "540", "${orderInfo.consignorStoreName}")
                PrinterHelper.Align(PrinterHelper.LEFT)
                PrinterHelper.SetBold("2")
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "45", "580", "收件信息:")
                PrinterHelper.Align(PrinterHelper.LEFT)
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "45", "620", "${orderInfo.receiveStoreName}")
                PrinterHelper.SetBold("1")
                PrinterHelper.Align(PrinterHelper.LEFT)
                PrinterHelper.Text(PrinterHelper.TEXT, "4", "0", "45", "660", "${orderInfo.receivePhone}")
                PrinterHelper.AutLine("45", "700", 500, 4, false, false, "${orderInfo.receiveAddress}")
////            //恢复默认设置
                PrinterHelper.SetMag("1", "1")
                PrinterHelper.SetBold("0")
                PrinterHelper.Form()
                isSuccess = isSuccess && PrinterHelper.Print() > 0
            }
        }


        return isSuccess
    }

}
